"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = generateCssBundles;

var _lodash = require("lodash");

var _babelTypes = require("babel-types");

var t = _interopRequireWildcard(_babelTypes);

var _ast = require("interlock/lib/util/ast");

var _getTemplate = require("./get-template");

var _getTemplate2 = _interopRequireDefault(_getTemplate);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) arr2[i] = arr[i]; return arr2; } else { return Array.from(arr); } }

const exportObjTmpl = (0, _getTemplate2.default)("export-object", node => t.program([node]));

/**
 * Replace CSS modules with JavaScript modules which reference the CSS
 * output bundle destination.
 *
 * @param  {Array}  bundles                  Original bundles array.
 * @param  {Array}  cssBundles               New CSS bundles.
 * @param  {Object} originBundleCssBundleMap Mapping of origin bundle to sibling
 *                                           CSS bundle.
 * @param  {Object} moduleClassnameMaps      Mapping of filename to friendly/module
 *                                           classname maps.
 *
 * @return {Array}                           Original bundles plus CSS bundles.
 */
function replaceCssModules(bundles, cssBundles, originBundleCssBundleMap, moduleClassnameMaps) {

  const cssBundlesStats = moduleClassnameMaps && cssBundles.map(bundle => {
    const moduleClassnames = bundle.modules.map(module => moduleClassnameMaps[module.path]);
    const combinedJson = _lodash.assign.apply(undefined, _toConsumableArray([{}].concat(moduleClassnames)));

    return {
      type: "css-stats",
      dest: `${ bundle.dest }.json`,
      moduleHashes: [],
      raw: JSON.stringify(combinedJson)
    };
  }) || [];

  return bundles.map((bundle, bIdx) => {
    if (!(bIdx in originBundleCssBundleMap)) {
      return bundle;
    }
    const cssBundle = cssBundles[originBundleCssBundleMap[bIdx]];

    return (0, _lodash.assign)({}, bundle, { modules: bundle.modules.map(module => {
        if (module.type !== "css") {
          return module;
        }

        const exportValue = moduleClassnameMaps && moduleClassnameMaps[module.path] ? (0, _ast.fromObject)((0, _lodash.assign)({}, moduleClassnameMaps[module.path], { _path: cssBundle.dest })) : t.stringLiteral(cssBundle.dest);

        return (0, _lodash.assign)({}, module, {
          type: "javascript",
          ast: exportObjTmpl({ EXPORT_VALUE: exportValue })
        });
      }) });
  }).concat(cssBundles, cssBundlesStats);
}

function generateCssBundles(bundles, moduleClassnameMaps) {
  const cssBundleSeeds = [];
  const originBundleCssBundleMap = {};

  /**
   * Extract out the CSS modules from each JavaScript bundle, placing each set
   * into a newly created CSS bundle.
   */
  bundles.forEach((bundle, bIdx) => {
    if (
    // Build dynamic CSS bundles for JavaScript bundles.
    bundle.type !== "javascript" &&
    // Build dynamic CSS bundles for CSS bundles that aren't entry-points.
    // This condition will occur when `interlock-css` is combined with
    // `interlock-node`, where a bundle is generated for every input module.
    !(bundle.type === "css" && !bundle.isEntryPt)) {
      return;
    }

    const cssModules = bundle.modules.filter(module => module.type === "css");

    if (cssModules.length) {
      originBundleCssBundleMap[bIdx] = cssBundleSeeds.length;

      cssBundleSeeds.push({
        type: "css",
        dest: "[hash].css",
        modules: cssModules,
        moduleHashes: cssModules.map(module => module.hash),
        isEntryPt: false
      });
    }
  });

  const cssBundlesPs = cssBundleSeeds.map(bundle => this.hashBundle(bundle).then(this.interpolateFilename));

  return Promise.all(cssBundlesPs).then(cssBundles => replaceCssModules(bundles, cssBundles, originBundleCssBundleMap, moduleClassnameMaps));
}
module.exports = exports['default'];