"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _slicedToArray = function () { function sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"]) _i["return"](); } finally { if (_d) throw _e; } } return _arr; } return function (arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return sliceIterator(arr, i); } else { throw new TypeError("Invalid attempt to destructure non-iterable instance"); } }; }();

exports.default = function () {
  let opts = arguments.length <= 0 || arguments[0] === undefined ? {} : arguments[0];

  const mode = opts.mode || "insert";
  const modules = !!opts.modules;
  const isCssFile = opts.filter || /\.css$/;

  const processor = (0, _postcss2.default)(opts.plugins || []);

  let moduleClassnameMaps = null;
  if (modules) {
    moduleClassnameMaps = {};
    processor.use((0, _postcssModules2.default)({
      getJSON(modulePath, json) {
        moduleClassnameMaps[modulePath] = json;
      }
    }));
  }

  checkMode(mode);

  return (override, transform) => {
    /**
     * Modules whose file paths match the provided filter (`.css` by default)
     * should be of type `css`.
     */
    transform("setModuleType", module => {
      return isCssFile.test(module.path) ? (0, _lodash.assign)({}, module, { type: "css" }) : module;
    });

    /**
     * Modules of type `css` should be parsed with PostCSS.
     */
    override("parseModule", module => {
      if (module.type !== "css") {
        return override.CONTINUE;
      }
      return (0, _lodash.assign)({}, module, {
        ast: _postcss2.default.parse(module.rawSource, {
          from: module.path
        })
      });
    });

    /**
     * CSS is transformed via a PostCSS processor.  This processor is
     * configured when the plugin is initialized, using the plugins that
     * were provided.
     */
    override("transformModule", module => {
      if (module.type !== "css") {
        return override.CONTINUE;
      }
      return processor.process(module.ast).then(result => {
        return (0, _lodash.assign)({}, module, {
          ast: result.root,
          synchronousRequires: []
        });
      });
    });

    /**
     * No extra work needs to occur here.  CSS modules should have no
     * run-time dependencies.
     */
    override("updateRequires", module => {
      if (module.type !== "css") {
        return override.CONTINUE;
      }
      return module;
    });

    /**
     * Bundles should be of type `css` if their module entry point is a CSS
     * module.
     */
    override("initBundle", bundleOpts => {
      if (!bundleOpts.module || bundleOpts.module.type !== "css") {
        return override.CONTINUE;
      }

      return (0, _lodash.assign)({}, bundleOpts, {
        type: "css"
      });
    });

    transform("generateBundles", function (bundles, _ref) {
      var _ref2 = _slicedToArray(_ref, 2);

      let moduleMaps = _ref2[1];

      // Return a JavaScript object with selectors as keys and rulesets as
      // values.  These rulesets will be expressed as objects with rule names
      // as keys and rule values as values.
      if (mode === "object") {
        return (0, _genRulesetObj2.default)(bundles);
      }

      // Output a .css file for per-bundle CSS modules.  If in CSS module
      // mode, instead return a mapping of original and mapped class names.
      if (mode === "bundle") {
        return _genCssBundles2.default.call(this, bundles, moduleClassnameMaps);
      }

      // Transforms CSS modules into separate distinct CSS output bundles, and
      // replaces with a JavaScript module.  When required, this module will
      // insert a <script> tag into the DOM, referencing the CSS output bundle.
      if (mode === "tag") {}
      // TODO


      // Transform CSS into JS that inserts the rules, and return the DOM element.
      if (mode === "insert") {
        return (0, _genStyleLoaders2.default)(bundles, (0, _lodash.values)(moduleMaps.byHash), moduleClassnameMaps);
      }

      return bundles;
    });

    /**
     * CSS modules that are to be emitted as part of a CSS bundle should not
     * be included in the URL hash used for loading JS modules.
     */
    transform("getUrls", urls => {
      return (0, _lodash.chain)(urls).map((url, moduleId) => [moduleId, url]).filter(_ref3 => {
        var _ref4 = _slicedToArray(_ref3, 2);

        let url = _ref4[1];
        return !isCssFile.test(url);
      }).fromPairs().value();
    });

    /**
     * No extra work needs to occur here.
     */
    override("constructBundle", bundle => {
      // `css-stats` bundles may be emitted when CSS modules output is enabled
      // and in `bundle` mode.
      if (bundle.type !== "css" || bundle.type !== "css-stats") {
        return override.CONTINUE;
      }
      return bundle;
    });

    /**
     * For any bundles of type `css`, convert the PostCSS ASTs of constituent
     * CSS modules to their string form and concatenate.
     */
    override("generateRawBundles", bundle => {
      // `css-stats` bundles may be emitted when CSS modules output is enabled
      // and in `bundle` mode.
      if (bundle.type === "css-stats") {
        return bundle;
      } else if (bundle.type !== "css") {
        return override.CONTINUE;
      }
      return (0, _lodash.assign)({}, bundle, {
        raw: bundle.modules.map(module => module.ast.toString()).join("\n")
      });
    });
  };
};

var _postcss = require("postcss");

var _postcss2 = _interopRequireDefault(_postcss);

var _postcssModules = require("postcss-modules");

var _postcssModules2 = _interopRequireDefault(_postcssModules);

var _lodash = require("lodash");

var _genStyleLoaders = require("./gen-style-loaders");

var _genStyleLoaders2 = _interopRequireDefault(_genStyleLoaders);

var _genCssBundles = require("./gen-css-bundles");

var _genCssBundles2 = _interopRequireDefault(_genCssBundles);

var _genRulesetObj = require("./gen-ruleset-obj");

var _genRulesetObj2 = _interopRequireDefault(_genRulesetObj);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const VALID_MODES = ["bundle", "insert", "object"];

const checkMode = mode => {
  if (!(0, _lodash.includes)(VALID_MODES, mode)) {
    throw new Error(`interlock-css: '${ mode }' is not a valid mode.  Must be one of: ${ VALID_MODES.join(", ") }.`); // eslint-disable-line max-len
  }
};

module.exports = exports['default'];